/* Copyright 1994, Roger Smith for Sterling Software and NASA-Ames Research Center
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of NASA and Sterling Software (collectively 
 * or individually) are not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  NASA and Sterling Software makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * NASA & STERLING SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL NASA & STERLING SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * 1994 Revision
 * Author:           Roger Smith, Sterling Software @ NASA-Ames Research Center
 *                   Moffett Field, California, rsmith@proteus.arc.nasa.gov
 */
/*
 * Network File System statistics
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
/*#include <sys/vfs.h>*/
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <nlist.h>
#include <fcntl.h>
#ifdef sun
#  include <sys/param.h>
#  include <mntent.h>
#endif
#ifndef mips
#  include <kvm.h>
#endif
#ifdef ultrix
#  include <sys/vm.h>
#endif
#include <string.h>
#include "is.h"

extern char *gprogname;
extern struct _sysStats sysStats;
extern struct _procStats procStats;

struct _rpcNnfs rpcNnfs;

struct nlist nl1[] = {
#define	X_RCSTAT	0
	{ "_rcstat" },
#define	X_CLSTAT	1
	{ "_clstat" },
#define	X_RSSTAT	2
	{ "_rsstat" },
#define	X_SVSTAT	3
	{ "_svstat" },
#define X_TOTAL         3
	{ "" },
};
#ifdef sun
kvm_t *kd;			/* kernel id from kvm_open */
#else
int   kmem;			/* for ultrix and sgi */
#endif

char *vmunix = NULL;		/* name for /vmunix */
char *core   = NULL;		/* name for /dev/kmem */
Bool firstTime = TRUE;		/* for init code */

static int	ccode, scode; 	/* Identify server and client code present */

getnfs(cflag, sflag, nflag, rflag)
int	cflag;		/* client stats */
int	sflag;		/* server stats */
int	nflag;		/* nfs stats */
int	rflag;		/* rpc stats */
{
    
    if (firstTime) {
	setup();
	if (sflag && (!scode)) {
	    fprintf(stderr, "%s: kernel is not configured with the server nfs and rpc code.\n", gprogname);
	}
	if (cflag && (!ccode)) {
	    fprintf(stderr, "%s: kernel is not configured with the client nfs and rpc code.\n", gprogname);
	}
	firstTime = FALSE;
    }
    getstats();
    return;
}

kio(rdwr, id, buf, len)
    int rdwr, id, len;
    char *buf;
{
    if (nl1[id].n_type == 0) {
	fprintf(stderr, "%s: '%s' not in namelist\n", gprogname,
		nl1[id].n_name);
	bzero(buf, len);
	return;
    }
    if (rdwr == 0) {
#ifdef sun
	if (kvm_read(kd, nl1[id].n_value, buf, len) != len) {
	    fprintf(stderr, "%s: kernel read error\n", gprogname);
	    exit(1);
	}
#endif
#ifdef ultrix
	lseek(kmem, (long)nl1[id].n_value, SEEK_SET);
	read(kmem, buf, len);
#endif
    }
}

#define kread(id, buf, len)  kio(0, id, (char *)(buf), len)
#define kwrite(id, buf, len) kio(1, id, (char *)(buf), len)

getstats()
{
	time(&rpcNnfs.time_stamp);
	if (ccode) {
		kread(X_RCSTAT, &rpcNnfs.rpc_c, sizeof rpcNnfs.rpc_c);
		kread(X_CLSTAT, &rpcNnfs.nfs_c, sizeof rpcNnfs.nfs_c);
	}
	if (scode) {
		kread(X_RSSTAT, &rpcNnfs.rpc_s, sizeof rpcNnfs.rpc_s);
		kread(X_SVSTAT, &rpcNnfs.nfs_s, sizeof rpcNnfs.nfs_s);
	}
}

setup()
{
    extern int errno;
#ifdef sun
	if ((kd = kvm_open(vmunix, core, NULL, O_RDONLY, gprogname)) == NULL)
		exit(1);
	if (kvm_nlist(kd, nl1) < 0) {
		fprintf(stderr, "%s: bad namelist\n", gprogname);
		exit(1);
	}
#else ifdef ultrix || sgi
	if (( kmem = open("/dev/kmem", 0)) < 0 ) {
	    perror("ERROR - opening \"/dev/kmem\"");
	    exit(errno);
	}
#ifdef ultrix
	nlist("/vmunix", nl1);
#else
	nlist("/unix", nl1);
#endif
#endif
	/* check to see if the client code is present */
	ccode = (nl1[X_RCSTAT].n_type != 0);
	scode = (nl1[X_RSSTAT].n_type != 0);
}
